"""
Internal module configuring ACE as a Django app.
"""
from django.apps import AppConfig


class EdxAceConfig(AppConfig):
    """
    Configuration for the edx_ace Django application.
    """

    name = 'edx_ace'
